﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace VamPyro{
	public class ColourMatch: MVRScript {

        #region versioning
        public string pluginAuthor = "VamPyro";
        public string pluginName = "ColourMatch";
        public string pluginVersion = "0.2";
        public string pluginDate = "06/01/2020";
        public string pluginDescription = "Match colour parameters across hair";
        #endregion

        public class HairColourSelector {
            public JSONStorableBool SelectionName { get; set; }
            public DAZHairGroup HairGroup { get; set; }
            public HairSimControl HairSim { get; set; }
            public DAZHairGroupControl HairGroupControl { get; set; }
            public UIDynamicToggle currentToggle;

            public HairColourSelector() { 
            }
        }

        #region Vars
        protected Atom person;
        
        protected List<HairColourSelector> sourceHairSelector;
        protected List<HairColourSelector> targetHairSelector;
        protected string cmText = "Colour Matcher v0.2\n(For Sim Hair V3 only)";
        protected bool updatingCheckBoxes = false;
        protected UIDynamicButton udbSelectAll;
        protected UIDynamicButton udbSelectNone;
        #endregion

        public override void Init() {
			try
            {
                pluginLabelJSON.val = "Hair colour matcher " + pluginVersion;
                person = SuperController.singleton.GetSelectedAtom();

                if (person.category != "People")
                {
                    Log("Not a person");
                    return;
                }
                sourceHairSelector = new List<HairColourSelector>();
                targetHairSelector = new List<HairColourSelector>();
                CreateUI();
            }
            catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

        private void RefreshHairChoices()
        {
            foreach (HairColourSelector hs in sourceHairSelector) {
                RemoveToggle(hs.currentToggle);
            }
            sourceHairSelector.Clear();
            foreach (HairColourSelector hs in targetHairSelector)
            {
                RemoveToggle(hs.currentToggle);
            }
            targetHairSelector.Clear();
            if (udbSelectAll != null)
                RemoveButton(udbSelectAll);
            if (udbSelectNone != null)
                RemoveButton(udbSelectNone);

            foreach (DAZHairGroup d in person.GetComponentsInChildren<DAZHairGroup>())
            {
                String infoString = "";
                if (d.displayName == "")
                    infoString = d.creatorName + " - ? ";
                else
                    infoString = d.creatorName + " - " + d.displayName;
                DAZHairGroupControl d1 = d.GetComponentInChildren<DAZHairGroupControl>();
                HairSimControl hs1 = d.GetComponentInChildren<HairSimControl>();
                if (d1 != null && hs1 != null) // valid SimV3 entry ? I think!
                {
                    HairColourSelector hsSource = new HairColourSelector();
                    HairColourSelector hsTarget = new HairColourSelector();

                    hsSource.SelectionName = new JSONStorableBool(infoString, false);
                    hsTarget.SelectionName = new JSONStorableBool(infoString, false);

                    hsSource.HairGroup = d;
                    hsSource.HairGroupControl = d1;
                    hsSource.HairSim = hs1;

                    hsTarget.HairGroup = d;
                    hsTarget.HairGroupControl = d1;
                    hsTarget.HairSim = hs1;
                    // custom callback for source
                    hsSource.SelectionName.setCallbackFunction += (bool b) => {
                        if (updatingCheckBoxes == true)
                            return;
                        updatingCheckBoxes = true;
                        foreach (HairColourSelector hsIterated in sourceHairSelector) {
                            if (hsIterated.SelectionName.val == true) {
                                hsIterated.SelectionName.val = false;
                            }
                        }
                        hsSource.SelectionName.val = b;
                        updatingCheckBoxes = false;                        
                    };
                    hsSource.currentToggle = CreateToggle(hsSource.SelectionName, false);
                    hsTarget.currentToggle = CreateToggle(hsTarget.SelectionName, true);
                    sourceHairSelector.Add(hsSource);
                    targetHairSelector.Add(hsTarget);
                }
                else
                {
                    //Log("No DHGC or HSC"); // not a SimV3 hair item
                }
            }
            udbSelectNone = CreateButton("<Clear all target hair>", false);
            udbSelectNone.button.onClick.AddListener(() => {
                        foreach (HairColourSelector hs in targetHairSelector)
                        {
                        hs.SelectionName.val = false;
                        }
                    }               
                );
            udbSelectAll = CreateButton("<Select all target hair>", true);
            udbSelectAll.button.onClick.AddListener(() => {
                        foreach (HairColourSelector hs in targetHairSelector)
                        {
                            hs.SelectionName.val = true;
                        }
                    }
                );
        }

        public void CreateUI() {
            JSONStorableString jsText = new JSONStorableString("help text", cmText);
            UIDynamicTextField udt = CreateTextField(jsText, false);

            udt.height = 115;

            UIDynamicButton udbRefresh = CreateButton("Refresh Hair Selection", true);
            udbRefresh.button.onClick.AddListener(() => RefreshHairChoices());
            UIDynamicButton udbUpdateHairCOlour = CreateButton("Update", true);
            udbUpdateHairCOlour.button.onClick.AddListener(() => UpdateHairSourceToTarget());
            UIDynamicTextField ut1 = CreateTextField(new JSONStorableString("label1", "\n         Select Source hair below"),false);
            UIDynamicTextField ut2 = CreateTextField(new JSONStorableString("label1", "\n    Select Target Hair below & press\n          update to copy values"),true);
            Color c = new Color(160f, 140f, 170f,0);
            ut1.height = 100;
            ut1.backgroundColor = c;
            ut1.useGUILayout = false;
            ut2.height = 100;
            ut2.backgroundColor = c;
            RefreshHairChoices();
            
        }

        public void UpdateHairSourceToTarget() {
            // get Source hair
            HairColourSelector hsSource =null;
            foreach (HairColourSelector hs in sourceHairSelector) {
                if (hs.SelectionName.val == true) {
                    hsSource = hs;
                    break;
                }
                    
            }
            if (hsSource != null) {
                HairSimControl hsc = hsSource.HairGroup.GetComponentInChildren<HairSimControl>();
                if (hsc != null) {
                    foreach (HairColourSelector hsTarget in targetHairSelector) {
                        if (hsTarget.SelectionName.val == true) {
                            /*
                             *  PN - rootColor
                                PN - tipColor
                                PN - specularColor
                                FPN  - colorRolloff
                                FPN  - diffuseSoftness
                                FPN  - primarySpecularSharpness
                                FPN  - secondarySpecularSharpness
                                FPN  - specularShift
                                FPN  - fresnelPower
                                FPN  - fresnelAttenuation
                                FPN  - randomColorPower
                                FPN  - randomColorOffset
                                FPN  - IBLFactor
                                FPN  - normalRandomize
                             */
                            hsTarget.HairSim.SetColorParamValue("rootColor", hsc.GetColorParamValue("rootColor"));
                            hsTarget.HairSim.SetColorParamValue("tipColor", hsc.GetColorParamValue("tipColor"));
                            hsTarget.HairSim.SetColorParamValue("specularColor", hsc.GetColorParamValue("specularColor"));
                            hsTarget.HairSim.SetFloatParamValue("colorRolloff", hsc.GetFloatParamValue("colorRolloff"));
                            hsTarget.HairSim.SetFloatParamValue("diffuseSoftness", hsc.GetFloatParamValue("diffuseSoftness"));
                            hsTarget.HairSim.SetFloatParamValue("primarySpecularSharpness", hsc.GetFloatParamValue("primarySpecularSharpness"));
                            hsTarget.HairSim.SetFloatParamValue("secondarySpecularSharpness", hsc.GetFloatParamValue("secondarySpecularSharpness"));
                            hsTarget.HairSim.SetFloatParamValue("specularShift", hsc.GetFloatParamValue("specularShift"));
                            hsTarget.HairSim.SetFloatParamValue("fresnelPower", hsc.GetFloatParamValue("fresnelPower"));
                            hsTarget.HairSim.SetFloatParamValue("fresnelAttenuation", hsc.GetFloatParamValue("fresnelAttenuation"));
                            hsTarget.HairSim.SetFloatParamValue("randomColorPower", hsc.GetFloatParamValue("randomColorPower"));
                            hsTarget.HairSim.SetFloatParamValue("randomColorOffset", hsc.GetFloatParamValue("randomColorOffset"));
                            hsTarget.HairSim.SetFloatParamValue("IBLFactor", hsc.GetFloatParamValue("IBLFactor"));
                            hsTarget.HairSim.SetFloatParamValue("normalRandomize", hsc.GetFloatParamValue("normalRandomize"));
                           
                        }
                    }
                }
            }
            
        }


		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

        void Log(string s)
        {
            SuperController.LogMessage(s);
        }
   }
}